@@ -217,27 +217,6 @@ class TestListAllocations(test_api_base.BaseApiTest):
         next_marker = data['allocations'][-1]['uuid']
         self.assertIn(next_marker, data['next'])
 
-    def test_collection_links_custom_fields(self):
-        cfg.CONF.set_override('max_limit', 3, 'api')
-        fields = 'uuid,extra'
-        allocations = []
-        for i in range(5):
-            allocation = obj_utils.create_test_allocation(
-                self.context,
-                node_id=self.node.id,
-                uuid=uuidutils.generate_uuid(),
-                name='allocation%s' % i)
-            allocations.append(allocation.uuid)
-
-        data = self.get_json(
-            '/allocations?fields=%s' % fields,
-            headers=self.headers)
-        self.assertEqual(3, len(data['allocations']))
-
-        next_marker = data['allocations'][-1]['uuid']
-        self.assertIn(next_marker, data['next'])
-        self.assertIn('fields', data['next'])
-
     def test_get_collection_pagination_no_uuid(self):
         fields = 'node_uuid'
         limit = 2
@@ -407,12 +386,197 @@ class TestPatch(test_api_base.BaseApiTest):
                                      'value': 'bar',
                                      'op': 'add'}],
                                    expect_errors=True,
-                                   headers=self.headers)
+                                   headers={api_base.Version.string: '1.56'})
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.METHOD_NOT_ALLOWED, response.status_int)
 
+    def test_update_not_found(self):
+        uuid = uuidutils.generate_uuid()
+        response = self.patch_json('/allocations/%s' % uuid,
+                                   [{'path': '/name', 'value': 'b',
+                                     'op': 'replace'}],
+                                   expect_errors=True,
+                                   headers=self.headers)
+        self.assertEqual(http_client.NOT_FOUND, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
 
-def _create_locally(_api, _ctx, allocation, _topic):
+    def test_add(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/extra/foo', 'value': 'bar',
+                                     'op': 'add'}], headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_int)
+
+    def test_add_non_existent(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/foo', 'value': 'bar',
+                                     'op': 'add'}],
+                                   expect_errors=True,
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertTrue(response.json['error_message'])
+
+    def test_add_multi(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/extra/foo1', 'value': 'bar1',
+                                     'op': 'add'},
+                                    {'path': '/extra/foo2', 'value': 'bar2',
+                                     'op': 'add'}], headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % self.allocation.uuid,
+                               headers=self.headers)
+        expected = {"foo1": "bar1", "foo2": "bar2"}
+        self.assertEqual(expected, result['extra'])
+
+    def test_replace_invalid_name(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/name', 'value': '[test]',
+                                     'op': 'replace'}],
+                                   expect_errors=True,
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertTrue(response.json['error_message'])
+
+    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_replace_singular(self, mock_utcnow, mock_notify):
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+
+        mock_utcnow.return_value = test_time
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/name',
+                                     'value': 'test', 'op': 'replace'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % self.allocation.uuid,
+                               headers=self.headers)
+        self.assertEqual('test', result['name'])
+        return_updated_at = timeutils.parse_isotime(
+            result['updated_at']).replace(tzinfo=None)
+        self.assertEqual(test_time, return_updated_at)
+        mock_notify.assert_has_calls([mock.call(mock.ANY, mock.ANY, 'update',
+                                      obj_fields.NotificationLevel.INFO,
+                                      obj_fields.NotificationStatus.START),
+                                      mock.call(mock.ANY, mock.ANY, 'update',
+                                      obj_fields.NotificationLevel.INFO,
+                                      obj_fields.NotificationStatus.END)])
+
+    def test_replace_name_with_none(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/name',
+                                     'value': None, 'op': 'replace'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % self.allocation.uuid,
+                               headers=self.headers)
+        self.assertIsNone(result['name'])
+
+    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(objects.Allocation, 'save')
+    def test_update_error(self, mock_save, mock_notify):
+        mock_save.side_effect = Exception()
+        allocation = obj_utils.create_test_allocation(self.context)
+        self.patch_json('/allocations/%s' % allocation.uuid, [{'path': '/name',
+                        'value': 'new', 'op': 'replace'}],
+                        expect_errors=True, headers=self.headers)
+        mock_notify.assert_has_calls([mock.call(mock.ANY, mock.ANY, 'update',
+                                      obj_fields.NotificationLevel.INFO,
+                                      obj_fields.NotificationStatus.START),
+                                      mock.call(mock.ANY, mock.ANY, 'update',
+                                      obj_fields.NotificationLevel.ERROR,
+                                      obj_fields.NotificationStatus.ERROR)])
+
+    def test_replace_multi(self):
+        extra = {"foo1": "bar1", "foo2": "bar2", "foo3": "bar3"}
+        allocation = obj_utils.create_test_allocation(
+            self.context, extra=extra, uuid=uuidutils.generate_uuid())
+        new_value = 'new value'
+        response = self.patch_json('/allocations/%s' % allocation.uuid,
+                                   [{'path': '/extra/foo2',
+                                     'value': new_value, 'op': 'replace'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % allocation.uuid,
+                               headers=self.headers)
+
+        extra["foo2"] = new_value
+        self.assertEqual(extra, result['extra'])
+
+    def test_remove_uuid(self):
+        response = self.patch_json('/allocations/%s' % self.allocation.uuid,
+                                   [{'path': '/uuid', 'op': 'remove'}],
+                                   expect_errors=True,
+                                   headers=self.headers)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+
+    def test_remove_singular(self):
+        allocation = obj_utils.create_test_allocation(
+            self.context, extra={'a': 'b'}, uuid=uuidutils.generate_uuid())
+        response = self.patch_json('/allocations/%s' % allocation.uuid,
+                                   [{'path': '/extra/a', 'op': 'remove'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % allocation.uuid,
+                               headers=self.headers)
+        self.assertEqual(result['extra'], {})
+
+        # Assert nothing else was changed
+        self.assertEqual(allocation.uuid, result['uuid'])
+
+    def test_remove_multi(self):
+        extra = {"foo1": "bar1", "foo2": "bar2", "foo3": "bar3"}
+        allocation = obj_utils.create_test_allocation(
+            self.context, extra=extra, uuid=uuidutils.generate_uuid())
+
+        # Removing one item from the collection
+        response = self.patch_json('/allocations/%s' % allocation.uuid,
+                                   [{'path': '/extra/foo2', 'op': 'remove'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % allocation.uuid,
+                               headers=self.headers)
+        extra.pop("foo2")
+        self.assertEqual(extra, result['extra'])
+
+        # Removing the collection
+        response = self.patch_json('/allocations/%s' % allocation.uuid,
+                                   [{'path': '/extra', 'op': 'remove'}],
+                                   headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.OK, response.status_code)
+        result = self.get_json('/allocations/%s' % allocation.uuid,
+                               headers=self.headers)
+        self.assertEqual({}, result['extra'])
+
+        # Assert nothing else was changed
+        self.assertEqual(allocation.uuid, result['uuid'])
+
+    def test_remove_non_existent_property_fail(self):
+        response = self.patch_json(
+            '/allocations/%s' % self.allocation.uuid,
+            [{'path': '/extra/non-existent', 'op': 'remove'}],
+            expect_errors=True, headers=self.headers)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_code)
+        self.assertTrue(response.json['error_message'])
+
+
+def _create_locally(_api, _ctx, allocation, topic):
+    if 'node_id' in allocation and allocation.node_id:
+        assert topic == 'node-topic', topic
+    else:
+        assert topic == 'some-topic', topic
     allocation.create()
     return allocation
 
@@ -427,6 +591,10 @@ class TestPost(test_api_base.BaseApiTest):
             fixtures.MockPatchObject(rpcapi.ConductorAPI, 'get_random_topic')
         ).mock
         self.mock_get_topic.return_value = 'some-topic'
+        self.mock_get_topic_for_node = self.useFixture(
+            fixtures.MockPatchObject(rpcapi.ConductorAPI, 'get_topic_for')
+        ).mock
+        self.mock_get_topic_for_node.return_value = 'node-topic'
 
     @mock.patch.object(notification_utils, '_emit_api_notification')
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
@@ -442,6 +610,7 @@ class TestPost(test_api_base.BaseApiTest):
         self.assertIsNone(response.json['node_uuid'])
         self.assertEqual([], response.json['candidate_nodes'])
         self.assertEqual([], response.json['traits'])
+        self.assertNotIn('node', response.json)
         result = self.get_json('/allocations/%s' % adict['uuid'],
                                headers=self.headers)
         self.assertEqual(adict['uuid'], result['uuid'])
@@ -449,6 +618,7 @@ class TestPost(test_api_base.BaseApiTest):
         self.assertIsNone(result['node_uuid'])
         self.assertEqual([], result['candidate_nodes'])
         self.assertEqual([], result['traits'])
+        self.assertNotIn('node', result)
         return_created_at = timeutils.parse_isotime(
             result['created_at']).replace(tzinfo=None)
         self.assertEqual(test_time, return_created_at)
@@ -551,7 +721,7 @@ class TestPost(test_api_base.BaseApiTest):
                                   headers=self.headers)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
         self.assertEqual('application/json', response.content_type)
-        self.assertTrue(response.json['error_message'])
+        self.assertIn('resource_class', response.json['error_message'])
 
     def test_create_allocation_resource_class_too_long(self):
         adict = apiutils.allocation_post_data()
@@ -636,15 +806,87 @@ class TestPost(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.METHOD_NOT_ALLOWED, response.status_int)
 
-    def test_create_with_node_uuid_not_allowed(self):
+    def test_create_node_uuid_not_allowed(self):
+        node = obj_utils.create_test_node(self.context)
         adict = apiutils.allocation_post_data()
-        adict['node_uuid'] = uuidutils.generate_uuid()
+        adict['node_uuid'] = node.uuid
         response = self.post_json('/allocations', adict, expect_errors=True,
                                   headers=self.headers)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['error_message'])
 
+    def test_backfill(self):
+        node = obj_utils.create_test_node(self.context)
+        adict = apiutils.allocation_post_data(node=node.uuid)
+        response = self.post_json('/allocations', adict,
+                                  headers=self.headers)
+        self.assertEqual(http_client.CREATED, response.status_int)
+        self.assertNotIn('node', response.json)
+        result = self.get_json('/allocations/%s' % adict['uuid'],
+                               headers=self.headers)
+        self.assertEqual(adict['uuid'], result['uuid'])
+        self.assertEqual(node.uuid, result['node_uuid'])
+        self.assertNotIn('node', result)
+
+    def test_backfill_with_name(self):
+        node = obj_utils.create_test_node(self.context, name='backfill-me')
+        adict = apiutils.allocation_post_data(node=node.name)
+        response = self.post_json('/allocations', adict,
+                                  headers=self.headers)
+        self.assertEqual(http_client.CREATED, response.status_int)
+        self.assertNotIn('node', response.json)
+        result = self.get_json('/allocations/%s' % adict['uuid'],
+                               headers=self.headers)
+        self.assertEqual(adict['uuid'], result['uuid'])
+        self.assertEqual(node.uuid, result['node_uuid'])
+        self.assertNotIn('node', result)
+
+    def test_backfill_without_resource_class(self):
+        node = obj_utils.create_test_node(self.context,
+                                          resource_class='bm-super')
+        adict = {'node': node.uuid}
+        response = self.post_json('/allocations', adict,
+                                  headers=self.headers)
+        self.assertEqual(http_client.CREATED, response.status_int)
+        result = self.get_json('/allocations/%s' % response.json['uuid'],
+                               headers=self.headers)
+        self.assertEqual(node.uuid, result['node_uuid'])
+        self.assertEqual('bm-super', result['resource_class'])
+
+    def test_backfill_copy_instance_uuid(self):
+        uuid = uuidutils.generate_uuid()
+        node = obj_utils.create_test_node(self.context,
+                                          instance_uuid=uuid,
+                                          resource_class='bm-super')
+        adict = {'node': node.uuid}
+        response = self.post_json('/allocations', adict,
+                                  headers=self.headers)
+        self.assertEqual(http_client.CREATED, response.status_int)
+        result = self.get_json('/allocations/%s' % response.json['uuid'],
+                               headers=self.headers)
+        self.assertEqual(uuid, result['uuid'])
+        self.assertEqual(node.uuid, result['node_uuid'])
+        self.assertEqual('bm-super', result['resource_class'])
+
+    def test_backfill_node_not_found(self):
+        adict = apiutils.allocation_post_data(node=uuidutils.generate_uuid())
+        response = self.post_json('/allocations', adict, expect_errors=True,
+                                  headers=self.headers)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+
+    def test_backfill_not_allowed(self):
+        node = obj_utils.create_test_node(self.context)
+        headers = {api_base.Version.string: '1.57'}
+        adict = apiutils.allocation_post_data(node=node.uuid)
+        response = self.post_json('/allocations', adict, expect_errors=True,
+                                  headers=headers)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+
 
 @mock.patch.object(rpcapi.ConductorAPI, 'destroy_allocation')
 class TestDelete(test_api_base.BaseApiTest):
