@@ -17,10 +17,8 @@ from oslo_utils import uuidutils
 from ironic.common import exception
 from ironic.common import neutron as neutron_common
 from ironic.common import states
-from ironic.common import utils as common_utils
 from ironic.conductor import task_manager
 from ironic.drivers.modules.network import common
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as obj_utils
 
@@ -31,8 +29,6 @@ class TestCommonFunctions(db_base.DbTestCase):
 
     def setUp(self):
         super(TestCommonFunctions, self).setUp()
-        self.config(enabled_drivers=['fake'])
-        mgr_utils.mock_the_extension_manager()
         self.node = obj_utils.create_test_node(self.context,
                                                network_interface='neutron')
         self.port = obj_utils.create_test_port(
@@ -434,13 +430,31 @@ class TestCommonFunctions(db_base.DbTestCase):
                 common.plug_port_to_tenant_network,
                 task, self.port)
 
+    @mock.patch.object(neutron_common, 'wait_for_host_agent', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    def test_plug_port_to_tenant_network_smartnic_port(
+            self, mock_gc, wait_port_mock, wait_agent_mock):
+        nclient = mock.MagicMock()
+        mock_gc.return_value = nclient
+        local_link_connection = self.port.local_link_connection
+        local_link_connection['hostname'] = 'hostname'
+        self.port.local_link_connection = local_link_connection
+        self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
+        self.port.is_smartnic = True
+        self.port.save()
+        with task_manager.acquire(self.context, self.node.id) as task:
+            common.plug_port_to_tenant_network(task, self.port)
+            wait_agent_mock.assert_called_once_with(
+                nclient, 'hostname')
+            wait_port_mock.assert_called_once_with(
+                nclient, self.vif_id, 'ACTIVE')
+
 
 class TestVifPortIDMixin(db_base.DbTestCase):
 
     def setUp(self):
         super(TestVifPortIDMixin, self).setUp()
-        self.config(enabled_drivers=['fake'])
-        mgr_utils.mock_the_extension_manager()
         self.interface = common.VIFPortIDMixin()
         self.node = obj_utils.create_test_node(self.context,
                                                network_interface='neutron')
@@ -664,8 +678,6 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
 
     def setUp(self):
         super(TestNeutronVifPortIDMixin, self).setUp()
-        self.config(enabled_drivers=['fake'])
-        mgr_utils.mock_the_extension_manager()
         self.interface = common.NeutronVIFPortIDMixin()
         self.node = obj_utils.create_test_node(self.context,
                                                network_interface='neutron')
@@ -963,10 +975,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         mock_get.assert_called_once_with(task, 'fake_vif_id')
         mock_clear.assert_called_once_with(self.port)
 
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
-    def test_port_changed_address(self, mac_update_mock, mock_warn):
+    def test_port_changed_address(self, mac_update_mock):
         new_address = '11:22:33:44:55:bb'
         self.port.address = new_address
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -974,7 +984,6 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mac_update_mock.assert_called_once_with(
                 self.port.extra['vif_port_id'], new_address,
                 context=task.context)
-            self.assertFalse(mock_warn.called)
 
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     def test_port_changed_address_VIF_MAC_update_fail(self, mac_update_mock):
@@ -1009,21 +1018,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             dhcp_update_mock.assert_called_once_with(
                 'fake-id', expected_dhcp_opts, context=task.context)
 
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    def test_port_changed_vif(self, dhcp_update_mock, mock_warn):
-        expected_extra = {'vif_port_id': 'new_ake-id', 'client-id': 'fake1'}
-        self.port.extra = expected_extra
-        with task_manager.acquire(self.context, self.node.id) as task:
-            self.interface.port_changed(task, self.port)
-            self.assertFalse(dhcp_update_mock.called)
-            self.assertEqual(1, mock_warn.call_count)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    def test_port_changed_extra_add_new_key(self, dhcp_update_mock, mock_warn):
+    def test_port_changed_extra_add_new_key(self, dhcp_update_mock):
         self.port.extra = {'vif_port_id': 'fake-id'}
         self.port.save()
         expected_extra = self.port.extra
@@ -1032,20 +1028,6 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.port_changed(task, self.port)
             self.assertFalse(dhcp_update_mock.called)
-            self.assertEqual(0, mock_warn.call_count)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    def test_port_changed_extra_no_deprecation_if_removing_vif(
-            self, dhcp_update_mock, mock_warn):
-        self.port.extra = {'vif_port_id': 'foo'}
-        self.port.save()
-        self.port.extra = {'foo': 'bar'}
-        with task_manager.acquire(self.context, self.node.id) as task:
-            self.interface.port_changed(task, self.port)
-            self.assertFalse(dhcp_update_mock.called)
-            self.assertEqual(0, mock_warn.call_count)
 
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
     def test_port_changed_client_id_fail(self, dhcp_update_mock):
@@ -1239,48 +1221,6 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.portgroup_changed(task, pg)
         self.assertFalse(mac_update_mock.called)
 
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
-    def test_update_portgroup_vif(self, mac_update_mock, mock_warn):
-        pg = obj_utils.create_test_portgroup(
-            self.context, node_id=self.node.id)
-        extra = {'vif_port_id': 'foo'}
-        pg.extra = extra
-        with task_manager.acquire(self.context, self.node.id) as task:
-            self.interface.portgroup_changed(task, pg)
-        self.assertFalse(mac_update_mock.called)
-        self.assertEqual(1, mock_warn.call_count)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
-    def test_update_portgroup_vif_removal_no_deprecation(self, mac_update_mock,
-                                                         mock_warn):
-        pg = obj_utils.create_test_portgroup(
-            self.context, node_id=self.node.id, extra={'vif_port_id': 'foo'})
-        pg.extra = {}
-        with task_manager.acquire(self.context, self.node.id) as task:
-            self.interface.portgroup_changed(task, pg)
-        self.assertFalse(mac_update_mock.called)
-        self.assertFalse(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
-    def test_update_portgroup_extra_new_key(self, mac_update_mock, mock_warn):
-        pg = obj_utils.create_test_portgroup(
-            self.context, node_id=self.node.id,
-            extra={'vif_port_id': 'vif-id'})
-        expected_extra = pg.extra
-        expected_extra['foo'] = 'bar'
-        pg.extra = expected_extra
-        with task_manager.acquire(self.context, self.node.id) as task:
-            self.interface.portgroup_changed(task, pg)
-        self.assertFalse(mac_update_mock.called)
-        self.assertFalse(mock_warn.called)
-        self.assertEqual(expected_extra, pg.extra)
-
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     def test_update_portgroup_address_fail(self, mac_update_mock):
         pg = obj_utils.create_test_portgroup(
@@ -1297,10 +1237,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mac_update_mock.assert_called_once_with(
                 'fake-id', new_address, context=task.context)
 
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
-    def test_update_portgroup_address_no_vif(self, mac_update_mock, mock_warn):
+    def test_update_portgroup_address_no_vif(self, mac_update_mock):
         pg = obj_utils.create_test_portgroup(
             self.context, node_id=self.node.id)
         new_address = '11:22:33:44:55:bb'
@@ -1309,7 +1247,6 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.portgroup_changed(task, pg)
         self.assertEqual(new_address, pg.address)
         self.assertFalse(mac_update_mock.called)
-        self.assertFalse(mock_warn.called)
 
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     def test_update_portgroup_nostandalone_ports_pxe_ports_exc(
